import {
  select,
  max,
  csv,
  scaleBand,
  scaleLinear,
  map
} from 'd3';

let data;
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

csv('data.csv').then((loadedData) => {
  data = loadedData;
  const columns = data.columns;
  data.forEach((d) => {
    columns.forEach((col) => {
      if(col == 'Continent')d[col] = d[col].slice(1)
      if((col =='Latitude') || (col == 'Longitude')){
        var temp = d[col].split('.');
        d[col] = temp[0]+'.'+temp[1]+temp[2];
        return;
      }

      d[col] = d[col].replaceAll(',','')

      if(d[col] == "")d[col] = "0";
      if(!isNaN(+d[col])){
        d[col] = +d[col];
      } 
      else if(d[col].includes('%')){
        d[col] = +(d[col].slice(0,-1));
      }
      else if(d[col].includes('$$')){
        d[col] = +(d[col].slice(1,-1));
      }
    })
  });
  
  var regions = d3.map(data, function(d){ return d.Continent; }).keys();
  d3.select("#regionDropdown")
    .selectAll("option")
    .data(regions)
    .enter()
    .append("option")
    .text(function(d) { return d; });

  var values = ["Population", "Agricultural Land( %)", "Land Area(Km2)", "Armed Forces size", "Birth Rate", "Co2-Emissions", "CPI", "Fertility Rate", "Forested Area (%)", "Gross primary education enrollment (%)", "Gross tertiary education enrollment (%)","Gasoline Price", "GDP", "Infant mortality", "Life expectancy", "Maternal mortality ratio", "Out of pocket health expenditure", "Physicians per thousand", "Population: Labor force participation (%)", "Tax revenue (%)", "Total tax rate", "Unemployment rate", "Urban_population" ];
  d3.select("#valueDropdown")
    .selectAll("option")
    .data(values)
    .enter()
    .append("option")
    .text(function(d) { return d; });
  
  // Set up initial chart
  if (data.length > 0) {
    const regions = map(data, d => d.Continent).keys();
    const values = ["Population", "Agricultural Land( %)", "Land Area(Km2)", "Armed Forces size", "Birth Rate", "Co2-Emissions", "CPI", "Fertility Rate", "Forested Area (%)", "Gross primary education enrollment (%)", "Gross tertiary education enrollment (%)", "Gasoline Price", "GDP", "Infant mortality", "Life expectancy", "Maternal mortality ratio", "Out of pocket health expenditure", "Physicians per thousand", "Population: Labor force participation (%)", "Tax revenue (%)", "Total tax rate", "Unemployment rate", "Urban_population" ];     
    updateChart(regions[0], values[0]);
  }

  // Event listeners for the dropdowns
  d3.select("#regionDropdown").on("change", function(d) {
    var selectedRegion = d3.select(this).property("value");
    var selectedValue = d3.select("#valueDropdown").property("value");
    updateChart(selectedRegion, selectedValue);
  });

  d3.select("#valueDropdown").on("change", function(d) {
    var selectedValue = d3.select(this).property("value");
    var selectedRegion = d3.select("#regionDropdown").property("value");
    updateChart(selectedRegion, selectedValue);
  });
  
  d3.selectAll('.button_click').on('click', function (d) {
    var buttonID = d3.select(this).attr('id');
    if (buttonID == 'asc') {
        update(
          data.sort(function (a, b) {
            return d3['ascending'](
              +a['Population'],
              +b['Population']
            );
          })
        );
      }

      if (buttonID == 'desc') {
        update(
          data.sort(function (a, b) {
            return d3['descending'](
              +a['Population'],
              +b['Population']
            );
          })
        );
      }
    }
  );
});

const colorScale = d3.scaleOrdinal(d3.schemeCategory10);
var margin = {
  left: 60,
  top: 10,
  bottom: 100,
  right: 10,
};

const x = scaleBand().range([
  margin.left,
  width - margin.right,
]);

var xAx = svg
  .append('g')
  .attr(
    'transform',
    'translate(' + margin.left + ',0)'
  );

const y = scaleLinear().range([
  height - margin.bottom,
  margin.top,
]);

var yAx = svg
  .append('g')
  .attr(
    'transform',
    'translate(' + margin.left + ',0)'
  );

function updateChart(selectedRegion, selectedValue) {
    // Filter data based on selected region
    var filteredData = data.filter(function(d) { 
      return d.Continent === selectedRegion; 
    });
    console.log(filteredData)
    // Update x-axis domain with countries from filtered data
    x.domain(filteredData.map((d) => d.Country));
    xAx.call(d3.axisBottom(x))
      .attr('transform', 'translate(0,' + (height - margin.bottom) + ')')
      .selectAll('text')
      .style('text-anchor', 'end')
      .attr('dx', '-1em')
      .attr('dy', '-0.5em')
      .attr('transform', 'rotate(-60)');

    // Update y-axis domain based on the selected value
    y.domain([0, max(filteredData, (d) => +d[selectedValue])]);
    yAx.call(d3.axisLeft(y).tickSize(-width));

    // Update the bars
    var bars = svg.selectAll('rect').data(filteredData);

    bars.enter()
      .append('rect')
      .merge(bars)
      .transition()
      .duration(1000)
      .attr('fill', (d, i) => colorScale(i))
      .attr('x', (d) => x(d.Country))
      .attr('y', (d) => y(+d[selectedValue]))
      .attr('width', x.bandwidth())
      .attr('height', (d) => y(0) - y(+d[selectedValue])); // Correct height calculation
      
    // Remove any excess bars
    bars.exit().remove();

    // Update the scatter plot
    var circles = svg.selectAll('circle').data(filteredData);

    circles.enter()
      .append('circle')
      .merge(circles)
      .transition()
      .duration(500)
      .attr('r', '3')
      .style('fill', 'blue')
      .attr('stroke', 'black')
      .attr('cx', (d) => x(d.Country) + x.bandwidth() / 2)
      .attr('cy', (d) => y(+d[selectedValue]));

    // Remove any excess circles
    circles.exit().remove();
};
